package com.ahai.vgk_game_services

import android.app.Activity
import android.content.Intent
import android.util.Log
import android.view.Gravity
import com.google.android.gms.auth.api.Auth
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.auth.api.signin.GoogleSignInAccount
import com.google.android.gms.auth.api.signin.GoogleSignInClient
import com.google.android.gms.auth.api.signin.GoogleSignInOptions
import com.google.android.gms.games.AchievementsClient
import com.google.android.gms.games.Games
import com.google.android.gms.drive.Drive
import com.google.android.gms.games.SnapshotsClient
import com.google.android.gms.games.snapshot.Snapshot
import com.google.android.gms.games.snapshot.SnapshotMetadataChange
import com.google.android.gms.games.LeaderboardsClient
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import io.flutter.plugin.common.PluginRegistry.ActivityResultListener
import java.io.IOException

import java.math.BigInteger
import java.util.Random

private const val CHANNEL_NAME = "vgk_game_services"
private const val RC_SIGN_IN = 9000
private const val SUCCESS = "success";
private const val ERROR = "error";
private const val RESPONSE = "response";
private const val MESSAGE = "message";
private const val ID = "id";
private const val NAME = "name";
private const val DESCRIPTION = "description";
private const val SNAPSHOT_DATA = "snapshotData";
private const val DISPLAY_NAME = "displayName";
private const val SCORE = "score";
private const val PERCENT = "percent";

private object Methods {
  const val getSignIn = "getSignIn"
  const val showLeaderboard = "showLeaderboard"
  const val submitScore = "submitScore"
  const val showAchievements = "showAchievements"
  const val unlockAchievement =  "unlockAchievement"
  const val setPercentAchievement = "setPercentAchievement"
  const val saveSnapShot = "saveSnapShot"
  const val loadSnapShot = "loadSnapShot"
}

class VgkGameServicesPlugin(private var activity: Activity? = null) : FlutterPlugin, MethodCallHandler, ActivityAware, ActivityResultListener {
  private var googleSignInClient: GoogleSignInClient? = null
  private var googleSignInAccount: GoogleSignInAccount? = null
  private var achievementClient: AchievementsClient? = null
  private var leaderboardsClient: LeaderboardsClient? = null
  private var snapshotsClient: SnapshotsClient? = null
  private var activityPluginBinding: ActivityPluginBinding? = null
  private var channel: MethodChannel? = null
  private var pendingOperation: PendingOperation? = null

  companion object {
    @JvmStatic
    fun registerWith(registrar: PluginRegistry.Registrar) {
      val channel = MethodChannel(registrar.messenger(), CHANNEL_NAME)
      val plugin = VgkGameServicesPlugin(registrar.activity())
      channel.setMethodCallHandler(plugin)
      registrar.addActivityResultListener(plugin)
    }
  }
 
  private class PendingOperation constructor(val method: String, val result: Result)

  private fun showLoginErrorIfNotLoggedIn(result: Result) {
    if (achievementClient == null || leaderboardsClient == null) {
      result.success("Please make sure to call signIn() first")
    }
  }

  override fun onAttachedToEngine(binding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(messenger, CHANNEL_NAME)
    channel?.setMethodCallHandler(this)
  }

  override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
    channel?.setMethodCallHandler(null)
    channel = null
  }

  override fun onDetachedFromActivity() {
    activityPluginBinding?.removeActivityResultListener(this)
    activityPluginBinding = null
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    onAttachedToActivity(binding)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activityPluginBinding = binding
    activity = binding.activity
    binding.addActivityResultListener(this)
  }

  override fun onDetachedFromActivityForConfigChanges() {
    onDetachedFromActivity()
  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
    if (requestCode == RC_SIGN_IN) {
      val result = Auth.GoogleSignInApi.getSignInResultFromIntent(data)
      val signInAccount = result?.signInAccount
      if (result?.isSuccess == true && signInAccount != null) {
        handleSignInResult(signInAccount)
      } else {
        var message = result?.status?.statusMessage ?: ""
        if (message.isEmpty()) {
          message = "Something went wrong " + result?.status
        }
        pendingOperation!!.result.error(ERROR, message, null)
        pendingOperation = null
      }
      return true
    }
    return false
  }

  private fun handleSignInResult(googleSignInAccount: GoogleSignInAccount) {
    val activity = this.activity!!
    val gamesClient = Games.getGamesClient(activity, GoogleSignIn.getLastSignedInAccount(activity)!!)
    gamesClient.setViewForPopups(activity.findViewById(android.R.id.content))
    gamesClient.setGravityForPopups(Gravity.TOP or Gravity.CENTER_HORIZONTAL)
    val playersClient = Games.getPlayersClient(activity!!, googleSignInAccount)
    playersClient.currentPlayer?.addOnSuccessListener { currentPlayer ->
      val successMap = mapOf(RESPONSE to SUCCESS,
              MESSAGE to "player connect to game center",
              ID to currentPlayer.playerId, DISPLAY_NAME to currentPlayer.displayName)

      pendingOperation!!.result.success(successMap)
      pendingOperation = null
    }?.addOnFailureListener {
      pendingOperation!!.result.success( "error fetching player profile")
      pendingOperation = null
    }
  }

  private fun getSignIn(result: Result) {
    val activity = activity ?: return
    val builder = GoogleSignInOptions.Builder(
            GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN)
    builder.requestScopes(Drive.SCOPE_APPFOLDER).requestId()
    // builder.requestScopes(Games.SCOPE_GAMES_SNAPSHOTS, Drive.SCOPE_APPFOLDER)
    googleSignInClient = GoogleSignIn.getClient(activity, builder.build())
    googleSignInClient?.silentSignIn()?.addOnCompleteListener { response ->
      pendingOperation = PendingOperation(Methods.getSignIn, result)
      if (response.isSuccessful) {
        googleSignInAccount = response.result
        handleSignInResult(googleSignInAccount!!)
      } else {
        Log.e(ERROR, "signInError", response.exception)
        Log.i("ExplicitSignIn", "Trying explicit sign in")
        explicitSignIn()
      }
    }
  }

  private fun explicitSignIn() {
    val activity = activity ?: return
    val builder = GoogleSignInOptions.Builder(
            GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN)
            .requestEmail()
    builder.requestScopes(Drive.SCOPE_APPFOLDER).requestId()
    // builder.requestScopes(Games.SCOPE_GAMES_SNAPSHOTS, Drive.SCOPE_APPFOLDER)
    googleSignInClient = GoogleSignIn.getClient(activity, builder.build())
    activity.startActivityForResult(googleSignInClient?.signInIntent, RC_SIGN_IN)
  }

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {

      Methods.getSignIn -> {
        getSignIn(result)
      }

      else -> result.notImplemented()
    }
  }
}